@@ -597,6 +597,11 @@ def impl_ebreak(inst: Simulation.Instruction, state: Simulation.ExecutionState,
     state.pc += 4
 
 
+def impl_mret(inst: Simulation.Instruction, state: Simulation.ExecutionState,
+              data_interface: Simulation.MemorySubsystem):
+    state.pc = state.old_pc
+
+
 ###################################################################################################
 # Nano RISC-V
 # ===========
@@ -610,6 +615,7 @@ def impl_ebreak(inst: Simulation.Instruction, state: Simulation.ExecutionState,
 #  S-Type(0x23,0x2):     SW     rs2, imm(rs1)     ; store word       mem[R[s1] + imm] <- R[s2]
 #                        SW     rs2, label        ;                  R[d] <- mem[R[0] + %lo(label)]
 #  0x00,0x10,0x00,0x73:  EBREAK                   ; stop execution
+#  0x30,0x10,0x00,0x73:  MRET                     ; return from interrupt
 #
 ###################################################################################################
 def instructions(mode: str):
@@ -637,6 +643,7 @@ def instructions(mode: str):
     ID_JAL = JType("JAL", 0x6f)
 
     ID_EBREAK = RawType("EBREAK", 0x00100073, impl_ebreak)
+    ID_MRET = RawType("MRET", 0x30000073, impl_mret)
 
     return [
         ID_ADD, ID_SUB, ID_AND, ID_OR, ID_SRA, ID_SRL, ID_SLL, ID_XOR,
@@ -644,5 +651,5 @@ def instructions(mode: str):
         ID_BEQ, ID_BNE, ID_BLT, ID_BGE,
         ID_ADDI, ID_LUI,
         ID_JALR, ID_JAL,
-        ID_EBREAK
+        ID_EBREAK, ID_MRET
     ]
