@@ -210,10 +210,11 @@ class InstDescription:
 #                               Processor Execution State
 ###################################################################################################
 class ExecutionState():
-    def __init__(self, registers: List[Optional[int]], reg_bits: int, max_pc: int, initial_pc: int):
+    def __init__(self, registers: List[Optional[int]], reg_bits: int, max_pc: int, initial_pc: int, old_pc: int):
         self.simulator = None
         self.end_of_execution = False
         self._pc = initial_pc
+        self._old_pc = old_pc
         self._registers = registers
         self.max_pc = max_pc
         self.reg_bits = reg_bits
@@ -235,6 +236,18 @@ class ExecutionState():
                                   self.simulator)
         self._pc = value
 
+    @property
+    def old_pc(self):
+        return self._old_pc
+
+    @old_pc.setter
+    def old_pc(self, value: int):
+        value = value & self._reg_mask
+        if value > self.max_pc:
+            raise SimulationError("Setting Old PC to {0} (0x{0:x}) is invalid.".format(value),
+                                  self.simulator)
+        self._old_pc= value
+
     def __getitem__(self, nr: int) -> int:
         value = self._registers[nr]
         if value is None:
@@ -293,8 +306,8 @@ class ExecutionState():
 
 class ZeroR0ExecutionState(ExecutionState):
     def __init__(self, reg_nr: int, reg_bit: int, max_pc: int, initial_pc: int,
-                 initial_value: Optional[int] = None):
-        super().__init__([0] + [initial_value] * (reg_nr - 1), reg_bit, max_pc, initial_pc)
+                 initial_value: Optional[int] = None, old_pc: Optional[int] = 0x00):
+        super().__init__([0] + [initial_value] * (reg_nr - 1), reg_bit, max_pc, initial_pc, old_pc)
 
     def __setitem__(self, nr: int, value: int):
         if nr == 0:
@@ -325,6 +338,9 @@ class PeripheralDevice:
     def format_state(self, skip_none: bool = False) -> List[str]:
         return []
 
+    def raise_interrupt(self, simulator: Optional['Simulator']) -> bool:
+        return False
+
 
 class MemorySubsystem:
     def __init__(self, simulator: Optional['Simulator']):
@@ -499,6 +515,9 @@ class Simulator:
             inst.desc.execute(inst, self.state, self.data_mem)
             for device in self._clocked_devices:
                 device.tick(self)
+                if(device.raise_interrupt(self)):
+                    self.state.old_pc = pc
+                    self.state.pc = 0x00
             self.instruction += 1
 
         for device in self._devices:
